# -*- coding: utf-8 -*-
from qgis.core import QgsProject, QgsFeatureRequest


cid_translator_table = {
    'owner': 'fr_owner_type',
    'manager': 'fr_manager_type',
    'ownership_origin': 'fr_ownershipe_origin_type',
    'legal_regime': 'fr_legal_regime_type',
    'rights': 'fr_rights_type',
    'promotion': 'fr_promotion_type',
    'assignment': 'fr_assignment_type',
    'usage': 'fr_usage_type',
    'conflict_type': 'fr_conflict_type_type',
    'conflicting_parts': 'fr_conflicting_parts_type',
}

class LoadedLayers:
    """Acces to the layers that are already open in the project"""

    def __init__(self, parent=None):
        self.project = QgsProject.instance()
        self.layers = dict()

        # FIXME: faire charger les couches par le plugin plutôt que
        # de dépendre du chargement des couches par le projet ?

    def get(self, schema, table):
        if (schema, table) in self.layers:
            return self.layers[(schema, table)]
        for l in self.project.mapLayers().values():
            if '"{}"."{}"'.format(schema, table) in l.source():
                self.layers[(schema, table)] = l
                return l
        print('Layer {}.{} not founded'.format(schema, table))
        return None

    def get_feature(self, schema, table, feature_id, id_key='id'):
        layer = self.get(schema, table)

        if layer:
            req = QgsFeatureRequest().setFilterExpression("{} = '{}'".format(id_key, feature_id))
            for f in layer.getFeatures(req):
                return f
        return None

    def get_features_by_cond(self, schema, table, select):
        # select(f) a function that return True if the feature f has to be selected
        layer = self.get(schema, table)
        features = []

        if layer is not None:
            for f in layer.getFeatures():
                if select(f):
                    features.append(f)
        return features

    def translate_en_to_fr(self, schema, table, value):
        f = self.get_feature(schema, table, value, id_key='en')
        if f:
            return f['fr']
        else:
            return value
