from .paggf_core import PAGGFCore, PAGGFAction
from . import paggf_version
from .ui.welcome import WelcomeEFDlg
from .ui.EFTF_action import EFTFEncodeDlg, EFTFPrintDlg


class PAGGF_EF(PAGGFCore):
    TITLE_PARAM = 'Paramétrage et maintenance BD EF/TF'
    TITLE_EF = 'Enquête foncière'
    TITLE_TF = 'Titre foncier'
    TITLE_SKETCH = 'Croquis orienté'

    def __init__(self, iface):
        super(PAGGF_EF, self).__init__(
            iface, paggf_version.EF, 'EF et Enregistrement TD')
        self.welcome_dlg_class = WelcomeEFDlg

        self.enable_paggf_action(
            'new_ef', {
                'menu_title': "Créer une enquête foncière pour parcelle",
                'tb_name': self.TITLE_EF,
                'pg_layers': [('gis', 'current_parcels')],
            })

        self.enable_paggf_action(
            'edit_efpf_data',
            {
                'menu_title': "Saisie EF/TF",
                'tb_name': self.TITLE_EF,
            }
        )

        self.enable_paggf_action(
            'generate_efpf',
            {
                'menu_title': 'Générer et imprimer EF/TF',
                'tb_name': self.TITLE_EF,
            }
        )

        self.enable_paggf_action(
            'select', {
                'menu_title': "Sélectionner une parcelle",
                'tb_name': self.TITLE_EF
            })

        self.enable_paggf_action('export_db', {
            'menu_title': "Exporter la base locale",
            'tb_name': self.TITLE_PARAM,
        })

        self.enable_paggf_action('sync_db', {
            'menu_title': "Synchroniser vers base centrale",
            'tb_name': self.TITLE_PARAM,
        })

        self.enable_paggf_action(
            'generate_and_validate_parcel_sketch',
            {
                'menu_title': "Préparer et valider un croquis orienté EF",
                'pg_layers': [('gis', 'ef_parcels_to_valid')],
                'tb_name': self.TITLE_SKETCH,
            }
        )

        self.enable_paggf_action(
            'download_parcel_sketch',
            {
                'menu_title': "Exporter un croquis EF",
                'pg_layers': [('gis', 'ef_parcels_to_valid')],
                "tb_name": self.TITLE_SKETCH,
            }
        )

        self.enable_paggf_action(
            'pv_upload_signatures',
            {
                'menu_title': 'Fusion EF et scan des signatures',
                'pg_layers': [('gis', 'ef_parcels_to_valid')],
                'tb_name': self.TITLE_EF,
            }
        )

    def toolbar_list_def(self):
        return [
            (self.TITLE_EF, self.TITLE_EF),
            (self.TITLE_PARAM, self.TITLE_PARAM),
            (self.TITLE_SKETCH, self.TITLE_SKETCH)
        ]

    def paggf_menu_content(self):
        return [  # DEFINITION DE L'ARBRE DU MENU
            # quand (nom, list) = sous menu
            # quand (nom, nom_action) = un access a une action
            PAGGFAction.actions['new_ef'],
            PAGGFAction.actions['edit_efpf_data'],
            PAGGFAction.actions['generate_efpf'],
            # 'Saisie Enquête foncière',
            'Saisie croquis orienté EF',
            # 'Saisie et impression d’un TF',
            (self.TITLE_SKETCH, [
                PAGGFAction.actions['generate_and_validate_parcel_sketch'],
                PAGGFAction.actions['download_parcel_sketch'],
            ]),
            (self.TITLE_PARAM, [
                'Paramétrage  BD EF',
                PAGGFAction.actions["export_db"],
                PAGGFAction.actions["sync_db"],
            ])
        ]

    def default_active_layer(self):
        return self.layers.get("gis", "parcels_to_valid")

    def edit_efpf_data(self):
        self.dlg = EFTFEncodeDlg(self.iface, parent=self.iface.mainWindow())
        self.dlg.show()

    def generate_efpf(self):
        self.dlg = EFTFPrintDlg(
            self.iface, parent=self.iface.mainWindow())
        self.dlg.show()
