import os.path
from PyQt5 import uic
from PyQt5.QtWidgets import QDialog
from PyQt5.QtGui import QDesktopServices
from PyQt5.QtCore import QUrl, QLocale
from ..paggf_utils import PAGGFUtils
from ..paggf_layers import LoadedLayers
from ..b64_conv import B64Convertor
from .. import paggf_version
from ..composer.ComposerValidation import ComposerValidation
from ..deps.num2words.num2words import num2words


class GenerateReportDlg(QDialog):
    def __init__(self, iface, layer, parcel, parent, output_filename):
        QDialog.__init__(self, parent)
        self.setupUi(self)
        self.parcel = parcel
        self.layer = layer
        self.layers = LoadedLayers()
        self.utils = PAGGFUtils(self.layers)
        self.iface = iface
        self.output_filename = output_filename

        self.fr_locale = QLocale(QLocale.French, QLocale.France)

        # Signal-slot connections
        self.generate_report.clicked.connect(self.generate_report_slot)

        self.download_unsigned_pv.clicked.connect(
            self.download_unsigned_report_slot)

        self.zoom_to_feature = True

    def set_template_file(self, template_file):
        self.template_path = os.path.join(self.utils.plugin_dir, template_file)

    def download_unsigned_report_slot(self):
        tmpfileurl = B64Convertor.file_from_base64(
            self.parcel[self.db_doc_report_field_name_pdf])
        QDesktopServices.openUrl(
            QUrl.fromLocalFile(tmpfileurl))

    def generate_report_slot(self):
        self.cv = ComposerValidation(
            self.parcel, self.template_path, self.iface,
            zoom_to_feature=self.zoom_to_feature,
            load_feature_in_sub_map=True,
            callback=self.callback_after_report_generation,
            output_pdf=True, output_png=False,
            output_filename=self.output_filename)

        self.cv.substitution_map['LOGO_BURUNDI_PATH_4HTML'] = os.path.join(
            '/', self.utils.plugin_dir, 'img/Blason_du_Burundi.svg')
        if(self.cv.substitution_map['LOGO_BURUNDI_PATH_4HTML'][0] != '/'):
            self.cv.substitution_map['LOGO_BURUNDI_PATH_4HTML'] = '/' + \
                self.cv.substitution_map['LOGO_BURUNDI_PATH_4HTML']

        self.cv.substitution_map['LOGO_BURUNDI_PATH'] = os.path.join(
            self.utils.plugin_dir, 'img/Blason_du_Burundi.svg')
        self.cv.substitution_map['NORTH_ARROW_PATH'] = os.path.join(
            self.utils.plugin_dir, 'img/north.png')

        self.add_specific_substitution()

        if self.report_type == paggf_version.CID:
            self.radio_button_form_generate()

        elif self.report_type == paggf_version.BM:
            self.vertices_angles_distances_table_generate()
            if self.parcel['cadastral_region']:
                self.cv.add_substitution('cadastral_region_or_other', self.parcel['cadastral_region_name'])
            else:
                self.cv.add_substitution('cadastral_region_or_other', self.parcel['other_cadastral_region'])

            self.cv.layers_config = {
                'carto': [
                    (self.layers.get('gis', 'parcels_to_valid'), os.path.join(
                        self.utils.plugin_dir, "layer_style/bm_parcel_A4.qml")),
                    (self.layers.get('gis', 'vertices_to_valid'), os.path.join(
                        self.utils.plugin_dir, "layer_style/bm_vertex_A4.qml")),
                    (self.layers.get('gis', 'parcel_buildings_to_valid'), os.path.join(
                        self.utils.plugin_dir, "layer_style/bm_building_A4.qml")),
                    (self.layers.get('gis', 'parcel_rivers_to_valid'), os.path.join(
                        self.utils.plugin_dir, "layer_style/bm_river_A4.qml")),
                    (self.layers.get('gis', 'parcel_roads_to_valid'), os.path.join(
                        self.utils.plugin_dir, "layer_style/bm_road_A4.qml")),
                    (self.layers.get('gis', 'current_parcels'), os.path.join(
                        self.utils.plugin_dir, "layer_style/bm_parcel_A4.qml")),
                    (self.layers.get('gis', 'current_vertices'), os.path.join(
                        self.utils.plugin_dir, "layer_style/bm_vertex_A4.qml")),
                    (self.layers.get('gis', 'current_parcel_buildings'), os.path.join(
                        self.utils.plugin_dir, "layer_style/bm_building_A4.qml")),
                    (self.layers.get('gis', 'current_parcel_rivers'), os.path.join(
                        self.utils.plugin_dir, "layer_style/bm_river_A4.qml")),
                    (self.layers.get('gis', 'current_parcel_roads'), os.path.join(
                        self.utils.plugin_dir, "layer_style/bm_road_A4.qml")),
                ]
            }

            self.cv.scale_step = 250
            self.cv.zoom_to_feature_scale = 1.25

        elif self.report_type == paggf_version.TF:
            delivery_date_day_month = 'NULL'
            delivry_date_year = 'NULL'

            if self.parcel['delivery_date']:
                delivery_date_day_month = self.fr_locale.toString(
                    self.parcel['delivery_date'], 'd MMMM')
                delivry_date_year = num2words(
                    int(self.parcel['delivery_date'].toString('yy')),
                    lang="fr")

            self.cv.add_substitution(
                'LOGO_BURUNDI_MARGE', os.path.join(
                    self.utils.plugin_dir, 'img/logo_burundi_marge.png')
            )
            self.cv.add_substitution(
                'delivery_date_day_month', delivery_date_day_month)
            self.cv.add_substitution('delivry_date_year', delivry_date_year)

        elif self.report_type == paggf_version.EF:
            self.cv.add_substitution(
                'BLASON_BURUNDI_NB_PATH',
                os.path.join(
                    self.utils.plugin_dir, 'img/Blason_du_Burundi_nb.png'))

            if self.parcel['land_district'] :
                land_district_for_report = self.layers.get_feature(
                    'property',
                    'land_district_enum',
                    self.parcel['land_district'])['name']
            else:
                land_district_for_report = self.parcel['land_district_other']
            self.cv.add_substitution(
                'land_district_for_report', land_district_for_report)

        self.hide()
        self.cv.exec_()

    def callback_after_report_generation(
            self, report_pdf_path, report_png_path):
        callback_pdf = self.cv.get_callback_to_store_into_feature(
            self.layer, self.parcel, self.db_doc_report_field_name_pdf, None
        )
        callback_pdf(report_pdf_path, report_png_path)

        self.parcel = self.layer.getFeature(self.parcel.id())

        self.update_display()
        self.show()

    def update_display(self):
        if self.parcel[self.db_doc_report_field_name_pdf]:
            self.report_generated_msg.show()
            self.report_not_generated_msg.hide()
            self.download_unsigned_pv.setEnabled(True)
        else:
            self.report_generated_msg.hide()
            self.report_not_generated_msg.show()
            self.download_unsigned_pv.setEnabled(False)

        self.repaint()

    def exec_(self):
        self.update_display()
        self.exec()

    def add_specific_substitution(self):
        pass


CID_FORM_CLASS, _ = uic.loadUiType(os.path.join(
    os.path.dirname(__file__), 'CID_generate_pv.ui'))


class CIDGenerateReportDlg(GenerateReportDlg, CID_FORM_CLASS):
    def __init__(self, iface, layer, parcel, parent=None):
        super(CIDGenerateReportDlg, self).__init__(
            iface, layer, parcel, parent, '{}_PVID_NON_SIGNE'.format(parcel['idc_parcel_code']))

        self.report_type = paggf_version.CID
        self.set_template_file('composer/cid_inventory.qpt')
        self.db_doc_report_field_name_pdf = "doc_generated_report"

    def radio_button_form_generate(self):
        if self.report_type == paggf_version.CID:
            # config of form in radio buttun. items arre (field_name, choices)
            # the composer_variable_name is field_name + "_radio_button_form"
            radio_button_form_config = [
                ("owner", [
                    (['State'], 'État'),
                    (['Municipality'], 'Commune'),
                    (['PublicInstitution'], 'Établissement public')]),
                    # (['Undefined', None], 'Non défini')]),
                ("manager", [
                    (['State'], 'État'),
                    (['Municipality'], 'Commune'),
                    ([
                        'PublicInstitution'],
                        'Etablissement public / société de droit public<br>'),
                    (['LegalPerson'], 'Personne morale de droit privé'),
                    (['PublicPerson'], 'Personne physique'),
                    (['Other'], 'Autre')]),
                ("ownership_origin", [
                    (['Transfer'], 'Transfert'),
                    (['Buying'], 'Achat'),
                    (['Donation'], 'Don et legs'),
                    (['Expropriation'], 'Exportation'),
                    (['State'], 'Propre à l\'Etat<br>'),
                    (['Recovery'],  'Terre reprise (après cession/concession)'),
                    (['Other'], 'Autre')]),
                ("legal_regime", [
                    (['Public'], 'Domaine public'),
                    (['Private'], 'Domaine privé')]),
                ("rights", [
                    (['Easement'], 'Servitude'),
                    (['Housing'], 'Usage d\'habitation'),
                    (['Emphyteusis'], 'Emphytéose'),
                    (['Mortgage'], 'Hypothèque'),
                    (['Rent'],  'Location'),
                    (['None'], 'Aucun')]),
                ("promotion", [
                    (['Total'], 'Totale'),
                    (['Partial'], 'Partielle'),
                    (['None', None], 'Non exploitée')]),
                ("assignment", [
                    (['Afforestation'], 'Boisement'),
                    (['Pasture'], 'Prairie'),
                    (['Market'], 'Marché'),
                    (['SecondarySchool'], 'Ecole secondaire'),
                    (['PrimarySchool'], 'Ecole primaire<br>'),
                    (['Cemetery'], 'Cimetière'),
                    (['PlayArea'], 'Terrain de jeux'),
                    (['BatwaVillage'], 'Village Batwa'),
                    (['HealthCentre'], 'Centre de santé<br>'),
                    (['NaturalForest'], 'Foret naturelle'),
                    (['NationalPark'], 'Parc national'),
                    (['NaturalReserve'], 'Réserve naturelle'),
                    (['Swamp'], 'Marais'),
                    (['Other'], 'Autre')]),
                ("usage", [
                    (['Afforestation'], 'Boisement'),
                    (['Pasture'], 'Prairie'),
                    (['Market'], 'Marché'),
                    (['SecondarySchool'], 'Ecole secondaire'),
                    (['PrimarySchool'], 'Ecole primaire<br>'),
                    (['Cemetery'], 'Cimetière'),
                    (['PlayArea'], 'Terrain de jeux'),
                    (['BatwaVillage'], 'Village Batwa'),
                    (['HealthCentre'], 'Centre de santé<br>'),
                    (['NaturalForest'], 'Foret naturelle'),
                    (['NationalPark'], 'Parc national'),
                    (['NaturalReserve'], 'Réserve naturelle'),
                    (['Swamp'], 'Marais'),
                    (['Other'], 'Autre')]),
                ("conflict_type", [
                    (['None'], 'Pas de conflit'),
                    (['Boundary', 'BoundaryAndOwnership'], 'Conflit de limite'),
                    (['Ownership', 'BoundaryAndOwnership'], 'Conflit de propriété')]),
                ("conflicting_parts", [
                    (['None'], 'Aucune'),
                    (['StateIndividual'], 'Etat et particulier'),
                    (['StatePublicInstit'], 'Etat et établissement public'),
                    (['StateMunicipality'], 'Etat et commune<br>'),
                    ([
                        'PublicInstitPublicInstit'],
                        'Etablissement public et établissement public'),
                    ([
                        'StateLegalPerson'],
                        'Etat et personne morale de droit privé<br>'),
                    ([
                        'PublicInstitMunicipality'],
                        'Etablissement public et commune')]),
            ]

            for field_name, choices in radio_button_form_config:
                composer_variable_name = "{}_radio_button_form".format(
                    field_name)
                self.cv.substitution_map[composer_variable_name] = \
                    self.get_html_radio_button_form(field_name, choices)

    def get_html_radio_button_form(self, fieldname, choices):
        html = "<form class='radio_choices'>\n"

        for values, label in choices:
            checked = ''
            if self.parcel[fieldname] in values:
                checked = 'checked'

            html += "<input type='radio' {}> {}\n".format(checked, label)

        html += '</form>'
        return html

BM_FORM_CLASS, _ = uic.loadUiType(os.path.join(
    os.path.dirname(__file__), 'BM_generate_pv.ui'))


class BMGenerateReportDlg(GenerateReportDlg, BM_FORM_CLASS):
    def __init__(self, iface, layer, parcel, parent=None):
        super(BMGenerateReportDlg, self).__init__(
            iface, layer, parcel, parent, '{}_PVBM'.format(parcel['idc_parcel_code']))

        self.report_type = paggf_version.BM
        self.set_template_file('composer/BM_report_A4.qpt')
        self.db_doc_report_field_name_pdf = "doc_surveying_report"

    def add_specific_substitution(self):
        if self.parcel['measuring_marking_date']:
            yy = int(self.parcel['measuring_marking_date'].toString('yy'))
            measuring_marking_y_word = num2words(yy, lang='fr')
            measuring_marking_dm_word = self.fr_locale.toString(
                self.parcel['measuring_marking_date'], 'd MMMM')
        else:
            measuring_marking_y_word = 'NULL'
            measuring_marking_dm_word = 'NULL'

        self.cv.add_substitution(
            'measuring_marking_y_word', measuring_marking_y_word)

        self.cv.add_substitution(
            'measuring_marking_dm_word', measuring_marking_dm_word)


        if self.parcel['end_date']:
            delivery_date_day_month = self.fr_locale.toString(
                self.parcel['end_date'], 'MMMM yyyy')
        else:
            delivery_date_day_month = 'NULL'

        self.cv.add_substitution(
            'place_and_date',
            "{}, {}".format(
                self.parcel['municipality_name'], delivery_date_day_month))
        self.cv.add_substitution(
            'calculated_area_m2_for_sketch',
            "{:.2f}".format(self.parcel['calculated_area_m2']))

        calculated_area_ha = int(self.parcel['calculated_area_m2']) // 10000
        ha_word = 'hectare'
        if calculated_area_ha > 1:
            ha_word += 's'

        calculated_area_a = (int(self.parcel['calculated_area_m2']) % 10000) // 100
        a_word = 'are'
        if calculated_area_a > 1:
            a_word += 's'

        calculated_area_ca = int(self.parcel['calculated_area_m2']) // 100
        ca_word = 'centiare'
        if calculated_area_ca > 1:
            ca_word += 's'

        calculated_area_in_word = "{} {} {} {} {} {}".format(
            num2words(calculated_area_ha, lang="fr"), ha_word,
            num2words(calculated_area_a, lang="fr"), a_word,
            num2words(calculated_area_ca, lang="fr"), ca_word)
        calculated_area_in_word = calculated_area_in_word.capitalize()

        self.cv.add_substitution('calculated_area_ha', calculated_area_ha)
        self.cv.add_substitution('calculated_area_a', calculated_area_a)
        self.cv.add_substitution('calculated_area_ca', calculated_area_ca)
        self.cv.add_substitution(
            'calculated_area_in_word', calculated_area_in_word)

    def vertices_angles_distances_table_generate(self):
        html_content = "\
<table>\n\
    <thead>\n\
        <tr>\n\
            <th>Sommet du périmètre et leur description</th>\n\
            <th>Longueur des cotés réduite à l'horizon<br>(1)</th>\n\
            <th>Angles aux sommets<br><s>Grades</s>(2)<br>Degrés(2)<br>(1)</th>\n\
            <th colspan=\"2\">Autres renseignements éventuels permettant le \
                calcul de la superficie, l’établissement, l’orientation du \
                croquis (azimut ou gisement ou orientation d’un côté) ainsi \
                que le repérage des sommets</th>\n\
            <th>Description des côtés</th>\n\
            <th>Tenant et aboutissants</th>\n\
        </tr>\n\
    </thead>\n\
    <tbody>\n\
"

        l = self.layers.get("gis", "vertices_angles_distances")
        vertex_list = [
            v for v in l.getFeatures() if v['id_parcel'] == self.parcel['id']]

        for i, v in enumerate(vertex_list):
            v_point = v.geometry().asPoint()
            html_content += "\
        <tr>\n\
            <td>{}{}-{}</td>\n\
            <td>{:.2f}</td>\n\
            <td>{:.3f}</td>\n\
            <td>{:.2f}</td>\n\
            <td>{:.2f}</td>\n\
            {}\n\
        </tr>".format(
                ('P' if v['is_main'] else 'S'),
                v['parcel_num_at_creation'],
                v['vertex_num_at_creation'],
                v['distance'],
                v['angle'],
                v_point.x(),  # FIXME CHECK WITH AIME
                v_point.y(),  # FIXME CHECK WITH AIME
                ("\
            <td rowspan=\"{}\">\
                <div class=\"rotate\">Tous les côtés sont des droites</div>\
            </td>\
            <td rowspan=\"{}\">\
                <div class=\"rotate\">Voir les indications au croquis</div>\
            </td>".format(len(vertex_list), len(vertex_list)) if i == 0 else ""))

        html_content += "\
    </tbody>\n\
</table>\n\
"

        self.cv.substitution_map[
            'vertices_angles_distances_table'] = html_content


EF_FORM_CLASS, _ = uic.loadUiType(os.path.join(
    os.path.dirname(__file__), 'CID_generate_pv.ui'))


class EFGenerateReportDlg(GenerateReportDlg, EF_FORM_CLASS):
    def __init__(self, iface, layer, parcel, parent=None):
        super(EFGenerateReportDlg, self).__init__(
            iface, layer, parcel, parent, '{}_FE_NON_SIGNE'.format(parcel['cadastral_number']))

        self.report_type = paggf_version.EF
        self.set_template_file('composer/EF_report_A4_justify.qpt')
        self.db_doc_report_field_name_pdf = "doc_generated_report"
        self.zoom_to_feature = False


class TFGenerateReportDlg(GenerateReportDlg, EF_FORM_CLASS):
    def __init__(self, iface, layer, parcel, parent=None):
        super(TFGenerateReportDlg, self).__init__(
            iface, layer, parcel, parent, '{}_TF'.format(parcel['cadastral_number']))

        self.report_type = paggf_version.TF
        self.set_template_file('composer/TF_report_A4.qpt')
        self.db_doc_report_field_name_pdf = "doc_report"
