from PyQt5 import uic
from PyQt5.QtWidgets import QDialog
from qgis.core import QgsFeatureRequest
import os.path
from ..paggf_layers import LoadedLayers
from ..paggf_utils import PAGGFUtils, get_background_layer
from ..composer.ComposerValidation import ComposerValidation
from .select_hill import SelectHillDlg

FORM_CLASS, _ = uic.loadUiType(os.path.join(
    os.path.dirname(__file__), 'select_hill.ui'))

def hill_sketch_dlg(iface):
    dlg = SelectHillDlg()
    if dlg.exec_() == QDialog.Rejected:
        return

    layers = LoadedLayers()
    utils = PAGGFUtils(layers)

    template_path = os.path.join(
        utils.plugin_dir, 'composer/cid_sketch_A3_hill_map.qpt')

    feature = layers.get_feature('ref', 'hill_districts', dlg.hill_id())

    cv = ComposerValidation(
        feature, template_path, iface, zoom_to_feature=True,
        output_png=True, scale_step=250)

    cv.add_substitutions({
        'LOGO_GIZ_PATH': os.path.join(utils.plugin_dir, 'img/logo_giz.png'),
        'LOGO_EUROPE_PATH': os.path.join(utils.plugin_dir, 'img/logo_europe.png'),
        'LOGO_PAGGF_PATH': os.path.join(utils.plugin_dir, 'img/logo_paggf.png'),
        'LOGO_BURUNDI_PATH':os.path.join(utils.plugin_dir, 'img/Blason_du_Burundi.svg'),
        'NORTH_ARROW_PATH': os.path.join(utils.plugin_dir, 'img/north.png'),
        'province': dlg.province_name(),
        'municipality': dlg.municipality_name(),
        'hill': dlg.hill_name(),
    })

    cv.layers_config = {
        'carto': [
            (layers.get('gis', 'encoded_idc_parcels'), os.path.join(
                utils.plugin_dir, "layer_style/encoded_idc_parcels_hill_map.qml")),
            (layers.get('ref', 'provinces'), None),
            (layers.get('ref', 'municipalities'), None),
            (get_background_layer(), None)
        ],
    }

    return cv
