from .load_attr_from_shp import LoadAttrFromFile

class LoadBMAttrFromFile(LoadAttrFromFile):
    def __init__(self, feature_to, layer_to, parent=None):
        LoadAttrFromFile.__init__(self, feature_to, layer_to, parent)

    def fields_map(self):
        return [
            # (field_name_in_shp, field_name_in_db)
            # (field_name_in_shp, field_name_in_db, enum_map, other_field)
            ('RBM', 'commission_reference'),
            ('NBRNE', 'num_installed_boundaries'),
            ('INSTR', 'measuring_instrument'),
            ('OBS', 'observations_other'),
            ('OBS_BM', 'observations_boundaries'),
            ('PRMORC', 'parcelling_plan')
        ]

    def optional_fields_map(self):
        return [
            # (field_name_in_shp, field_name_in_db)
            # (field_name_in_shp, field_name_in_db, enum_map, other_field)
            ('DATE', 'begin_date'),
            ('DATE', 'end_date'),
            ('DATE_MES', 'measuring_date'),
            ('DATE_BORN', 'marking_date'),
            ('BRN_CASSE', 'num_broken_boundaries'),
            ('PROP', 'owner', {"etat": 'State',
                               "commune" : "Municipality",
                               "etablissement public" : "PublicInstitution",
                               # other field
                       None: "Other"}, "owner_other"),
            ('GEST', 'manager', {'etat':'State',
                                 'personne physique': 'NaturalPerson',
                                 "personne morale de droit prive": "LegalPerson",
                                 "commune": "Municipality",
                                 "etablissement public": "PublicInstitution",
                                 None: "Other"}, "manager_other"),
            ('OD', 'ownership_origin', {"propre à l'etat": 'State',
                                        "achat": "Buying",
                                        "cession": "Recovery",
                                        "don et leg": "Donation",
                                        "expropriation": "Expropriation",
                                        "echange": "Transfer",
                                        None: "Other"}, "origin_other"),
            ('USA', 'usage', {"boisement": "Afforestation",
                              "centre de sante": "HealthCenre",
                              "cimetière": "Cemetery",
                              "cimétière": "Cemetery",
                              "ecofo": "NaturalForest",
                              "ecole secondaire": "SecondarySchool",
                              "marais": "Swamp",
                              "marché": "Market",
                              "prairie": "Pasture",
                              "terrain de jeux": "PlayArea",
                              None: "Other"}, "usage_other"),
            ('RJ', 'legal_regime', {'domaine privé' : 'Private', 'domaine public': 'Public'}, None),
            ('DC', 'rights', {'location' : 'Rent',
                              'amphythéose': 'Emphyteusis',
                              'habitation': 'Housing',
                              'servitude': 'Easement',
                              'hypothèque': 'Mortgage',
                              None: 'None'}, None),
            ('EM', 'promotion', {'non exploité': 'None',
                                 'partiel': 'Partial',
                                 'total': 'Total',
                                 None: 'None'}, None),
            ('AF', 'assignment', {'boisement' : 'Afforestation',
                                  'prairie': 'Pasture',
                                  'marché': 'Marché',
                                  'ecole secondaire': 'SecondarySchool',
                                  'ecole primaire': 'PrimarySchool',
                                  'cimetière': 'Cemetery',
                                  'cimétière': 'Cemetery',
                                  'terrain de jeux': 'PlayArea',
                                  'village batwa': 'BatwaVillage',
                                  'centre de sante': 'HealthCentre',
                                  'forêt naturelle': 'NaturalForest',
                                  'parc national': 'NationalPark',
                                  'réserve naturelle': 'NaturalReserve',
                                  'marais': 'Swamp',
                                  None: 'Other'}, "assignment_other"),
            ('OC', 'conflict_type', {'pas de conflit': 'None',
                                     'conflit de limite' : 'Boundary',
                                     'conflit de propriété' : 'Ownership',
                                     'conflit de limite et propriété': 'BoundaryAndOwnership'}, None),
            ('PC', 'conflicting_parts', {'aucune': 'None',
                                         'etat et particulier': 'StateIndividual',
                                         'etat et personne morale de droit prive': 'StateLegalPerson',
                                         'etat et commune' : 'StateMunicipality',
                                         'etat et etablissement public' : 'StatePublicInstit',
                                         'etablissement public et etablissement public' : 'PublicInstitPublicInstit',
                                         'etablissement public et commune' : 'PublicInstitMunicipality',
                                         None: 'None'}, None),
        ]
